#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include <time.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "pind.h"

Display *display ;
int screen ;
Window window ;

Font myfont;
XFontStruct *myfontstr;
GC gc_text;
XColor fg, bg;

unsigned long keyack_pixel, white_pixel ;

int total,elaps;
Bool live,prev;

void mysigterm();


void Out(void)
{
	XUndefineCursor(display, window);
	XFlush(display);
	exit(1);
}

/* ----------------------------------------------------------------------*/

void InitColors(void)
{
    keyack_pixel = BlackPixel( display, screen ) ;
    white_pixel = WhitePixel( display, screen ) ;

	bg.pixel=keyack_pixel;
	fg.pixel=white_pixel;
	XQueryColor(display, DefaultColormap(display, screen), &bg);
	XQueryColor(display, DefaultColormap(display, screen), &fg);
}

void creatGCs(void)
{
	gc_text=XCreateGC(display, window, 0, 0);
	XSetForeground(display, gc_text, keyack_pixel);
	XSetBackground(display, gc_text, white_pixel);
	XSetFont(display, gc_text, myfont);
}

void Init(char *disp_name, int argc, char *argv[])
{
	XEvent ev ;
	char *deffont = "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*";

	display = XOpenDisplay( disp_name ) ;
    if( display == (Display *)NULL )	{
		fprintf( stderr, "server display error !\n" ) ;
		exit( 1 ) ;
    	}
		
    screen = DefaultScreen( display ) ;
	InitColors();

    window = DefaultRootWindow( display ) ;

    myfont = XLoadFont( display, deffont ) ;
	if(!(myfontstr = XQueryFont( display, myfont )))
		Out();

	creatGCs();
}

/* ----------------------------------------------------------------------*/

void mysigterm()
{
	printf("\nXPIndC terminated...\n");
	Out();
}

Bool readStatus(void)
{
	FILE *filep;

	if(!(filep=fopen(STATUS_FILE, "rb")))
		return False;

	live=getw(filep);
	prev=getw(filep);
	elaps=getw(filep);
	total=getw(filep);

	fclose(filep);
	return True;
}

void calc(int time, int *h, int *m, int *s)
{
	*h=time/3600;
	*m=(time-(*h)*3600)/60;
	*s=time-(*h)*3600-(*m)*60;
}

void createCursor(char *buf1, char *buf2)
{
	Pixmap cursor, mask;
	int th, tm, ts, w, h, H, len1, len2;
	Cursor xcursor;
	GC gc_cursor, gc_mask;

	len1=strlen(buf1);
	len2=strlen(buf2);

	w=XTextWidth(myfontstr, buf1, len1);
	H=myfontstr->ascent + myfontstr->descent;
	h=H * 2 + 2;

	cursor=XCreatePixmap(display, window, w, h, 1);
	mask=XCreatePixmap(display, window, w, h, 1);

	gc_cursor=XCreateGC(display, cursor, 0, NULL);
	gc_mask=XCreateGC(display, mask, 0, NULL);

	XSetForeground(display, gc_cursor, 0);
	XSetForeground(display, gc_mask, 0);
	XSetFont(display, gc_cursor, myfont);
	XSetFont(display, gc_mask, myfont);
	XFillRectangle(display, cursor, gc_cursor, 0, 0, w, h);
	XFillRectangle(display, mask, gc_cursor, 0, 0, w, h);

	XSetForeground(display, gc_cursor, 1);
	XSetForeground(display, gc_mask, 1);

	XDrawString(display, cursor, gc_cursor, 0, myfontstr->ascent, buf1, len1);
	XDrawString(display, mask, gc_mask, 0, myfontstr->ascent, buf1, len1);
	XDrawString(display, cursor, gc_cursor, 0, H + 2 + myfontstr->ascent, buf2, len2);
	XDrawString(display, mask, gc_mask, 0, H + 2 + myfontstr->ascent, buf2, len2);

	xcursor=XCreatePixmapCursor(display, cursor, mask, &fg, &bg, 0, 0);
	XDefineCursor(display, window, xcursor);
	
	XFlush(display);
	XFreeCursor(display, xcursor);

	XFreeGC(display, gc_cursor);
	XFreeGC(display, gc_mask);
	XFreePixmap(display, cursor);
	XFreePixmap(display, mask);
}

void main(int argc, char *argv[])
{
    char *display_name=NULL;
	char buf1[20], buf2[20];
	int th, tm, ts;
	Bool ear;

    Init(display_name, argc, argv);

	signal(SIGTERM,mysigterm);
	signal(SIGHUP,mysigterm);
	signal(SIGINT,mysigterm);

	while(1)	{
		ear= (readStatus() ? True : False);

		if(ear)	{
			calc(elaps, &th, &tm, &ts);
			sprintf(buf1, "%02d:%02d:%02d", th, tm, ts);
			calc(total, &th, &tm, &ts);
			sprintf(buf2, "%02d:%02d:%02d", th, tm, ts);
			}
		else	{
			strcpy(buf1, "no server");
			buf2[0]='\0';
			}

		createCursor(buf1, buf2);
		sleep(3);
		}

	Out();
}
